@@ -19,6 +19,7 @@ from . import parse
 from .utils import meshgrid, slice_syntax, _is_query, _new_attribute_label
 from ._py3k_compat import genfromstr, iteritems, csv_reader, string_type
 from .schema_utils import change_axis_schema
+from .robust import join
 
 __all__ = ["sdbtype", "SciDBArray", "SciDBDataShape"]
 
@@ -131,6 +132,9 @@ class sdbtype(object):
     def __str__(self):
         return self.schema
 
+    def __eq__(self, other):
+        return str(self) == str(other)
+
     @property
     def names(self):
         return [f[0] for f in self.full_rep]
@@ -156,6 +160,13 @@ class sdbtype(object):
             pass
         return '<{0}>'.format(schema)
 
+    @classmethod
+    def from_full_rep(cls, rep):
+        schema = '<%s>' % (','.join('%s: %s %s' %
+                          (nm, typ, 'NULL' if null else '')
+            for nm, typ, null in rep))
+        return cls(schema)
+
     @classmethod
     def _schema_to_list(cls, schema):
         """Convert a scidb type schema to a list representation
@@ -1265,7 +1276,7 @@ class SciDBArray(object):
         self, other = disambiguate(self, other)
 
         f = self.afl
-        joined = f.join(self, other)
+        joined = join(self, other)
         att = _new_attribute_label('condition', joined)
         expr = "{self} {op} {other}".format(self=self.att_names[0],
                                             op=operator,
@@ -1289,7 +1300,7 @@ class SciDBArray(object):
 
         f = self.afl
         self, mask = disambiguate(self, mask)
-        joined = f.join(self, mask)
+        joined = join(self, mask)
         expr = '%s=TRUE' % mask.att_names[0]
         idx = _new_attribute_label('__idx', self, mask)
         return f.project(f.unpack(f.filter(joined, expr), idx),
