@@ -27,20 +27,10 @@ needs_scipy = pytest.mark.skipif(MISSING_SP, reason='Test requires SciPy')
 from scidbpy import SciDBArray, SciDBShimInterface, connect, SciDBDataShape
 from scidbpy.schema_utils import disambiguate
 
-sdb = connect()
+from . import sdb, TestBase, teardown_function
 RTOL = 1E-6
 
 
-def teardown_function(function):
-    sdb.reap()
-
-
-class TestBase(object):
-
-    def teardown_method(self, method):
-        teardown_function(method)
-
-
 def test_copy_rename():
     """Test the copying and renaming of an array"""
     X = sdb.random(10)
@@ -299,72 +289,6 @@ def test_slicing():
     #(slice(8, 2, -1), slice(7, 3, -1))
 
 
-def test_ops():
-    from operator import add, sub, mul, truediv, mod, pow
-
-    def check_join_op(op):
-        A = sdb.random((5, 5))
-        B = 1.2
-        C = op(A, B)
-        assert_allclose(C.toarray(), op(A.toarray(), B), rtol=RTOL)
-
-    for op in (add, sub, mul, truediv, mod, pow):
-        yield check_join_op, op
-
-
-def test_reverse_ops():
-    from operator import add, sub, mul, truediv, mod, pow
-
-    def check_join_op(op):
-        A = 1.2
-        B = sdb.random((5, 5))
-        C = op(A, B)
-        assert_allclose(C.toarray(), op(A, B.toarray()), rtol=RTOL)
-
-    for op in (add, sub, mul, truediv, mod, pow):
-        yield check_join_op, op
-
-
-def test_join_ops():
-    from operator import add, sub, mul, truediv, mod, pow
-
-    def check_join_op(op):
-        A = sdb.random((5, 5))
-        B = sdb.random((5, 5))
-        C = op(A, B)
-        assert_allclose(C.toarray(), op(A.toarray(), B.toarray()), rtol=RTOL)
-
-    for op in (add, sub, mul, truediv, mod, pow):
-        yield check_join_op, op
-
-
-def test_join_ops_same_array():
-    from operator import add, sub, mul, truediv, mod, pow
-
-    def check_join_op(op):
-        A = sdb.random((5, 5))
-        C = op(A, A)
-        assert_allclose(C.toarray(), op(A.toarray(), A.toarray()), rtol=RTOL)
-
-    for op in (add, sub, mul, truediv, mod, pow):
-        yield check_join_op, op
-
-
-def test_array_broadcast():
-    def check_array_broadcast(shape1, shape2):
-        A = sdb.random(shape1)
-        B = sdb.random(shape2)
-        C = A + B
-        assert_allclose(C.toarray(), A.toarray() + B.toarray())
-
-    for shapes in [((5, 1), 4), (4, (5, 1)),
-                   ((5, 1), (1, 4)), ((1, 4), (5, 1)),
-                   ((5, 1), (5, 5)), ((5, 5), (5, 1)),
-                   ((1, 5, 1), 4), (4, (1, 5, 1)),
-                   ((5, 1, 3), (4, 1)), ((4, 1), (5, 1, 3))]:
-        yield check_array_broadcast, shapes[0], shapes[1]
-
-
 def test_abs():
     A = sdb.random((5, 5))
     B = abs(A - 1)
@@ -480,10 +404,8 @@ def test_regrid():
 
 def test_bad_url():
 
-    with pytest.raises(ValueError) as cm:
+    with pytest.raises(Exception):
         SciDBShimInterface('http://www.google.com')
-    assert cm.value.args[0] == ('Could not connect to a SciDB instance at '
-                                'http://www.google.com')
 
 
 def test_random_persistent():
@@ -708,7 +630,7 @@ def test_toarray_smallchunk():
     assert_allclose(x.toarray(), expected)
 
 
-class TestAttributeAccess(object):
+class TestAttributeAccess(TestBase):
 
     def test_single(self):
         x = sdb.arange(5)
